# Xiid Enterprise Update Commandlet - Xiid Corporation 2024 #

# Temporary Variables
# TODO: Add logic for override of managev3 for self-hosted collectors
$managementPortal = "downloadv3.xiid.com"
#if (ManagementOverrideFlagGPO) {
#    $managementPortal = "(manageportalurl)/manage/assets/version/"
#}

# Check general paths
$xiidUserExists = Test-Path HKCU:"\Software\Xiid Corporation"
$xiidMachineExists = Test-Path HKLM:"\Software\Xiid Corporation"

$xiidSettingsUserExists = $false
if ($xiidUserExists) { $xiidSettingsUserExists = Test-Path HKCU:"\Software\Xiid Corporation\Settings" }

$xiidSettingsMachineExists = $false
if ($xiidMachineExists) { $xiidSettingsMachineExists = Test-Path HKLM:"\Software\Xiid Corporation\Settings" }

if (!$xiidSettingsUserExists -and !$xiidSettingsMachineExists) { exit(0) }

New-Item -Path "c:\" -Name "xiidtemp" -ItemType "directory"

if ($xiidSettingsUserExists) {
    # Xiid Wrapper Logic
    $wrapperInstallExists = $null -ne (Get-Item -Path HKCU:"\Software\Xiid Corporation\Settings").GetValue("XiidIMWrapperInstallStrategyType")

    $wrapperUpdateExists = $null -ne (Get-Item -Path HKCU:"\Software\Xiid Corporation\Settings").GetValue("XiidIMWrapperUpdateStrategyType")

    if ($wrapperInstallExists -or $wrapperUpdateExists) {

        Invoke-WebRequest https://$managementPortal/xiid-im-rdp-wrapper.json -OutFile C:\xiidtemp\wrapper_info.json
        $wrapperJson = (Get-Content 'C:\xiidtemp\wrapper_info.json' | Out-String | ConvertFrom-Json)
        $currentWrapperRelease = $wrapperJson.version
        $currentWrapperDownload = $wrapperJson.download
        
        Write-Host "Current Wrapper Release Version: $currentWrapperRelease"

        if ($wrapperInstallExists) {
            #    Install
            $wrapperFolderExists = Test-Path "C:\Program Files\Xiid.IM RDP Wrapper"

            $doWrapperInstall = $false

            $wrapperInstallSetting = (Get-Item -Path HKCU:"\Software\Xiid Corporation\Settings").GetValue("XiidIMWrapperInstallStrategyType")

            $doWrapperInstall = $wrapperInstallSetting -ne 0

            if ($doWrapperInstall) {
                # Do Wrapper Installation Logic Here
                Write-Output "Should Install Wrapper: $doWrapperInstall"
                if (!$wrapperFolderExists) {
                    if ($wrapperInstallSetting -eq 1) {
                        # Do Automatic Installation of Wrapper with latest version
                        Write-Host "Downloading current version of Wrapper from $currentWrapperDownload"
                        Invoke-WebRequest $currentWrapperDownload -OutFile C:\xiidtemp\wrapper-install.exe
                        (C:\xiidtemp\wrapper-install.exe /VERYSILENT /SUPPRESSMSGBOXES /NORESTART /custominstall true) 2>&1
                        Write-Host "Xiid RDP Wrapper successfully installed!" -ForegroundColor Green
                    }
                    elseif ($wrapperInstallSetting -eq 2) {
                        # Install from specific version
                        $wrapperInstallVersion = (Get-Item -Path HKCU:"\Software\Xiid Corporation\Settings").GetValue("XiidIMWrapperInstallStrategyPinnedVersion")
                        $wrapperDownload = "https://downloadv3.xiid.com/xiid-im-rdp-wrapper-setup-$wrapperInstallVersion.exe"
                        Write-Host "Downloading wrapper from $wrapperDownload"
                        Invoke-WebRequest $wrapperDownload -OutFile C:\xiidtemp\wrapper-install.exe
                        (C:\xiidtemp\wrapper-install.exe /VERYSILENT /SUPPRESSMSGBOXES /NORESTART /custominstall true) 2>&1
                        Write-Host "Xiid RDP Wrapper successfully installed!" -ForegroundColor Green
                    }
                }
                else {
                    Write-Host "Wrapper Folder already exists. Skipping Installation." -ForegroundColor Green
                }
            }
        }
        if ($wrapperUpdateExists) {
            #    Update

            $doWrapperUpdateLatest = $false

            $doWrapperUpdateLatest = (Get-Item -Path HKCU:"\Software\Xiid Corporation\Settings").GetValue("XiidIMWrapperUpdateStrategyType") -eq 1

            $doWrapperUpdatePinned = $false

            $doWrapperUpdatePinned = (Get-Item -Path HKCU:"\Software\Xiid Corporation\Settings").GetValue("XiidIMWrapperUpdateStrategyType") -eq 2

            if ($doWrapperUpdateLatest) {
                # Do Wrapper Update to Latest
                Write-Output "Should Update Wrapper: $doWrapperUpdateLatest"
                # Do Automatic Installation of Wrapper with latest version
                Write-Host "Downloading current version of Wrapper from $currentWrapperDownload"
                Invoke-WebRequest $currentWrapperDownload -OutFile C:\xiidtemp\wrapper-install.exe
                (C:\xiidtemp\wrapper-install.exe /VERYSILENT /SUPPRESSMSGBOXES /NORESTART /custominstall true) 2>&1
                Write-Host "Xiid RDP Wrapper successfully Updated!" -ForegroundColor Green
            }
            if ($doWrapperUpdatePinned) {
                # Do wrapper update to pinned version
                $wrapperInstallVersion = (Get-Item -Path HKCU:"\Software\Xiid Corporation\Settings").GetValue("XiidIMWrapperUpdateStrategyPinnedVersion")
                $wrapperDownload = "https://downloadv3.xiid.com/xiid-im-rdp-wrapper-setup-$wrapperInstallVersion.exe"
                Write-Host "Downloading wrapper from $wrapperDownload"
                Invoke-WebRequest $wrapperDownload -OutFile C:\xiidtemp\wrapper-install.exe
                (C:\xiidtemp\wrapper-install.exe /VERYSILENT /SUPPRESSMSGBOXES /NORESTART /custominstall true) 2>&1
                Write-Host "Xiid RDP Wrapper successfully Updated!" -ForegroundColor Green
            }
        }
    }
}

if ($xiidSettingsMachineExists) {
    # Xiid IM Agent

    $imAgentUpdateExists = $null -ne (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidIMAgentUpdateStrategyType")

    if ($imAgentUpdateExists) {
        $imAgentFolderExists = Test-Path "C:\Program Files\Xiid.IM Agent"
        if (!$imAgentFolderExists) {
            Write-Host "Error! Attempted to update IM Agent but could not installation folder." -ForegroundColor Red
        }
        else {
            $doIMAgentUpdateLatest = $false
            $doIMAgentUpdateLatest = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidIMAgentUpdateStrategyType") -eq 1

            $doIMAgentUpdatePinned = $false
            $doIMAgentUpdatePinned = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidIMAgentUpdateStrategyType") -eq 2

            $imAgentInfo = (C:\"Program Files\Xiid.IM Agent\IMAgent.exe" version) 2>&1 | % { "$_" }
            $imAgentInfo = $imAgentInfo -join " "
            $imVersionStartIndex = $imAgentInfo.IndexOf("Version:") + 8
            $imVersionEndIndex = $imAgentInfo.IndexOf("[")
            $imAgentVersion = $imAgentInfo.Substring($imVersionStartIndex, ($imVersionEndIndex - $imVersionStartIndex))
            $imAgentVersion = $imAgentVersion.Trim()
            if ($doIMAgentUpdateLatest) {
                Write-Output "Should Update IM Agent to latest: $doIMAgentUpdateLatest"
                # IM Agent Update to latest
                # Get current IM Agent Version
                Invoke-WebRequest https://$managementPortal/xiid-im-agent.json -OutFile C:\xiidtemp\im_agent_info.json
                $imAgentJson = (Get-Content 'C:\xiidtemp\im_agent_info.json' | Out-String | ConvertFrom-Json)
                $isAgentOutdated = $imAgentVersion -lt $imAgentJson.version
                if ($isAgentOutdated) {
                    # Do Update Logic Here
                    Write-Host "Updating IM Agent to version: $($imAgentJson.version)"
                    Invoke-WebRequest $imAgentJson.download -OutFile C:\xiidtemp\imagent-install.exe
                    Invoke-Command C:\xiidtemp\imagent-install.exe /VERYSILENT /noprompt
                    Write-Host "Updated IM Agent to version: $($imAgentJson.version)" -ForegroundColor Green
                }
                else {
                    Write-Host "Installed IM Agent version: $imAgentVersion | Current Release Version: $($imAgentJson.version) | No update required!" -ForegroundColor Green
                }
            }
            if ($doIMAgentUpdatePinned) {
                # Update IM Agent to pinned version
                $imAgentPinnedVersion = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidIMAgentUpdateStrategyPinnedVersion")
                $isAgentOutdated = $imAgentVersion -lt $imAgentPinnedVersion
                if($isAgentOutdated) {
                    $imAgentDownload = "https://downloadv3.xiid.com/xiid-im-agent-setup-$imAgentPinnedVersion.exe"
                    Write-Host "Downloading IM Agent from $imAgentDownload"
                    Invoke-WebRequest $imAgentDownload -OutFile C:\xiidtemp\im-agent-install.exe
                    (C:\xiidtemp\im-agent-install.exe /VERYSILENT /SUPPRESSMSGBOXES /NORESTART /custominstall true) 2>&1
                    Write-Host "Xiid IM Agent successfully updated!" -ForegroundColor Green
                }
                else {
                    Write-Host "Installed IM Agent version: $imAgentVersion | Pinned Version: $imAgentPinnedVersion | No update required!" -ForegroundColor Green
                }
            }
        }
    }

    # Xiid RDP Agent
    #    Install
    $rdpAgentInstallExists = $null -ne (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidRDPAgentInstallStrategyType")

    $doRDPAgentInstall = $false

    if ($rdpAgentInstallExists) { $doRDPAgentInstall = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidRDPAgentInstallStrategyType") -eq 1 }

    if ($doRDPAgentInstall) {
        # RDP Agent Install logic here
        # TODO: Add Later
        Write-Output "Should Install RDP Agent: $doRDPAgentInstall"
    }

    #    Update

    $rdpAgentUpdateExists = $null -ne (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidRDPAgentUpdateStrategyType")

    if ($rdpAgentUpdateExists) {
        $doRDPAgentUpdateLatest = $false
        $doRDPAgentUpdateLatest = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidRDPAgentUpdateStrategyType") -eq 1

        $doRDPAgentUpdatePinned = $false
        $doRDPAgentUpdatePinned = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidRDPAgentUpdateStrategyType") -eq 2

        $rdpAgentFolderExists = Test-Path "C:\Program Files\Xiid.IM RDP Agent"
        if (!$rdpAgentFolderExists) {
            Write-Host "Error! Attempted to update RDP Agent but could not find installation folder." -ForegroundColor Red
        }
        else {
            $rdpAgentInfo = (C:\"Program Files\Xiid.IM RDP Agent\RDPagent.exe" version) 2>&1 | % { "$_" }
            $rdpAgentInfo = $rdpAgentInfo -join " "
            $rdpVersionStartIndex = $rdpAgentInfo.IndexOf("Version:") + 10
            $rdpVersionEndIndex = $rdpAgentInfo.IndexOf("Built") - 1
            $rdpAgentVersion = $rdpAgentInfo.Substring($rdpVersionStartIndex, ($rdpVersionEndIndex - $rdpVersionStartIndex))
            $rdpAgentVersion = $rdpAgentVersion.Trim()
            if ($doRDPAgentUpdateLatest) {
                # RDP Agent Update to latest
                Write-Output "Should Update RDP Agent to latest: $doRDPAgentUpdateLatest"
                # Get current RDP Agent Version
                Invoke-WebRequest https://$managementPortal/xiid-im-rdp.json -OutFile C:\xiidtemp\rdp_agent_info.json
                $rdpAgentJson = (Get-Content 'C:\xiidtemp\rdp_agent_info.json' | Out-String | ConvertFrom-Json)
                $isAgentOutdated = $rdpAgentVersion -lt $rdpAgentJson.version
                if ($isAgentOutdated) {
                    # Do Update Logic Here
                    Write-Host "Updating RDP Agent to version: $($rdpAgentJson.version)"
                    Invoke-WebRequest $rdpAgentJson.download -OutFile C:\xiidtemp\rdpagent-install.exe
                    Invoke-Command C:\xiidtemp\rdpagent-install.exe /VERYSILENT /noprompt
                    Write-Host "Updated RDP Agent to version: $($rdpAgentJson.version)" -ForegroundColor Green
                }
                else {
                    Write-Host "Installed RDP Agent version: $rdpAgentVersion | Current Release Version: $($rdpAgentJson.version) | No update required!" -ForegroundColor Green
                }
            }
            if ($doRDPAgentUpdatePinned) {
                # RDP Agent Update to Pinned
                $rdpAgentPinnedVersion = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidRDPAgentUpdateStrategyPinnedVersion")
                $isAgentOutdated = $rdpAgentVersion -lt $rdpAgentPinnedVersion
                if($isAgentOutdated) {
                    $rdpAgentDownload = "https://downloadv3.xiid.com/xiid-im-rdp-setup-$rdpAgentPinnedVersion.exe"
                    Write-Host "Downloading RDP Agent from $rdpAgentDownload"
                    Invoke-WebRequest $rdpAgentDownload -OutFile C:\xiidtemp\rdp-agent-install.exe
                    (C:\xiidtemp\rdp-agent-install.exe /VERYSILENT /SUPPRESSMSGBOXES /NORESTART /custominstall true) 2>&1
                    Write-Host "Xiid RDP Agent successfully updated!" -ForegroundColor Green
                }
                else {
                    Write-Host "Installed RDP Agent version: $rdpAgentVersion | Pinned Version: $rdpAgentPinnedVersion | No update required!" -ForegroundColor Green
                }
            }
        }
    }

    # Xiid STLink
    # Install
    $stlinkInstallExists = $null -ne (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkInstallStrategyType")

    if ($stlinkInstallExists) {
        $installRSATTools = $false
        if ((Get-WindowsFeature -name RSAT-AD-Powershell).InstallState -eq "Installed") {
            Write-Host "AD Feature Already Installed."
        } else {
            Add-WindowsFeature RSAT-AD-PowerShell
            $installRSATTools = $true
        }
        # $installedADModule = $false
        # if (Get-Module -ListAvailable -Name ActiveDirectory) {
        #     Write-Host "AD Module already installed."
        # } 
        # else {
        #     Install-Module ActiveDirectory
        #     $installedADModule = $true
        # }
        # Import-Module ActiveDirectory

        $doSTLinkInstallLatest = $false
        $doSTLinkInstallLatest = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkInstallStrategyType") -eq 1

        $doSTLinkInstallPinned = $false
        $doSTLinkInstallPinned = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkInstallStrategyType") -eq 2

        $activateADAttribute = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkInstallStrategyCustomAttributeName")

        $activationCode = (Get-ADComputer -Identity $env:computername -Properties $activateADAttribute).$activateADAttribute

        Write-Host "Found activation code: $activationCode"

        if ($doSTLinkInstallLatest) {
            # STLink Install to latest
            Write-Output "Should Install latest STLink: $doSTLinkInstallLatest"
            Invoke-WebRequest https://$managementPortal/xiid-stlink.json -OutFile C:\xiidtemp\stlink_info.json
            $stlinkJson = (Get-Content 'C:\xiidtemp\stlink_info.json' | Out-String | ConvertFrom-Json)
            # Do install Logic Here
            Write-Host "Installing STLink version: $($stlinkJson.version)"
            $stlinkInstallationLink = "https://downloadv3.xiid.com/stlink-setup-windows-$($stlinkJson.version).exe"
            Invoke-WebRequest $stlinkInstallationLink -OutFile C:\xiidtemp\stlink-install.exe
            $params = "install -i $activationCode"
            $fullParams = $params.Split(" ")
            & "C:\xiidtemp\stlink-install.exe" $fullParams
            # Invoke-Command "C:\xiidtemp\stlink-install.exe" -ArgumentList 'install','-i',$activationCode
            Write-Host "Installed STLink version: $($stlinkJson.version)" -ForegroundColor Green
        }
        if ($doSTLinkInstallPinned) {
            # STLink install to pinned
            Write-Output "Should Install pinned STLink: $doSTLinkInstallPinned"
            $stlinkPinnedVersion = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkInstallStrategyPinnedVersion")
            Write-Host "Installing STLink version: $stlinkPinnedVersion"
            $stlinkInstallationLink = "https://downloadv3.xiid.com/stlink-setup-windows-$stlinkPinnedVersion.exe"
            Invoke-WebRequest $stlinkInstallationLink -OutFile C:\xiidtemp\stlink-install.exe
            $params = "install -i $activationCode"
            $fullParams = $params.Split(" ")
            & "C:\xiidtemp\stlink-install.exe" $fullParams
            Write-Host "Installed STLink pinned version: $stlinkPinnedVersion" -ForegroundColor Green
        }
        # if ($installedADModule) {
        #     Uninstall-Module ActiveDirectory
        # }
        if ($installRSATTools) {
            Remove-WindowsFeature RSAT-AD-Powershell
        }
    }

    # Update
    $stlinkUpdateExists = $null -ne (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkUpdateStrategyType")

    if ($stlinkUpdateExists) {
        $doSTLinkUpdateLatest = $false
        $doSTLinkUpdateLatest = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkUpdateStrategyType") -eq 1

        $doSTLinkUpdatePinned = $false
        $doSTLinkUpdatePinned = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkUpdateStrategyType") -eq 2

        $stlinkFolderExists = Test-Path "C:\Program Files\Xiid.ST Link"
        if (!$stlinkFolderExists) {
            Write-Host "Error! Attempted to update STLink but could not find installation folder." -ForegroundColor Red
        }
        else {
            $stlinkInfo = (C:\"Program Files\Xiid.ST Link\STlink.exe" version) 2>&1 | % { "$_" }
            $stlinkInfo = $stlinkInfo -join " "
            $stlinkVersionStartIndex = $stlinkInfo.IndexOf("Version:") + 8
            $stlinkVersionEndIndex = $stlinkInfo.IndexOf("[")
            $stlinkVersion = $stlinkInfo.Substring($stlinkVersionStartIndex, ($stlinkVersionEndIndex - $stlinkVersionStartIndex))
            $stlinkVersion = $stlinkVersion.Trim()
            if ($doSTLinkUpdateLatest) {
                # STLink Update logic here
                Write-Output "Should Update latest STLink: $doSTLinkUpdate"
                # Get current Stlink Agent Version
                Invoke-WebRequest https://$managementPortal/xiid-stlink.json -OutFile C:\xiidtemp\stlink_info.json
                $stlinkJson = (Get-Content 'C:\xiidtemp\stlink_info.json' | Out-String | ConvertFrom-Json)
                $isAgentOutdated = $stlinkVersion -lt $stlinkJson.version
                if ($isAgentOutdated) {
                    # Do Update Logic Here
                    Write-Host "Updating STLink to latest version: $($stlinkJson.version)"
                    $stlinkDownloadUrl = "https://downloadv3.xiid.com/stlink-setup-windows-$($stlinkJson.version).exe"
                    Invoke-WebRequest $stlinkDownloadUrl -OutFile C:\xiidtemp\stlink-install.exe
                    Invoke-Command C:\xiidtemp\stlink-install.exe install
                    Write-Host "Updated STlink to version: $($stlinkJson.version)" -ForegroundColor Green
                }
                else {
                    Write-Host "Installed STlink version: $stlinkVersion | Current Release Version: $($stlinkJson.version) | No update required!" -ForegroundColor Green
                }
            }
            if ($doSTLinkUpdatePinned) {
                # STLink Update logic here
                Write-Output "Should Update pinned STLink: $doSTLinkUpdate"
                $stlinkPinnedVersion = (Get-Item -Path HKLM:"\Software\Xiid Corporation\Settings").GetValue("XiidSTLinkInstallStrategyPinnedVersion")
                $isStlinkOutdated = $stlinkVersion -lt $stlinkPinnedVersion
                if($isStlinkOutdated) {
                    $stlinkDownload = "https://downloadv3.xiid.com/stlink-setup-windows-$stlinkPinnedVersion.exe"
                    Write-Host "Downloading STlink from $stlinkDownload"
                    Invoke-WebRequest $stlinkDownload -OutFile C:\xiidtemp\stlink-install.exe
                    (C:\xiidtemp\stlink-install.exe install) 2>&1
                    Write-Host "Xiid STlink successfully updated!" -ForegroundColor Green
                }
                else {
                    Write-Host "Installed STLink version: $stlinkVersion | Pinned Version: $stlinkPinnedVersion | No update required!" -ForegroundColor Green
                }
            }
        }
    }
}

Start-Sleep -Seconds 3
Remove-Item -LiteralPath "C:\xiidtemp" -Force -Recurse